#ifndef _IOSTREAM_
   #include <iostream>
#endif

#ifndef _EXCEPTION_
   #include <exception>
#endif

#include <cmath>

#ifndef _OSIFACE_
   #include "osiface.hpp"
#endif

#ifndef _IOPORT_
   #include "ioport.hpp"
#endif

using namespace std;

typedef struct {
        short int pitch;
        short int duration;
} NOTE;

//  Table of notes.  Given in half steps.  It's a communication from
// the "other side."

NOTE notes[] = {{14, 500}, {16, 500}, {12, 500}, {0, 500}, {7, 1000}};


//-----------------------------------------------------------------------------
//  Set PC's speaker frequency in Hz.  The speaker is controlled by an
//Intel 8253/8254 timer at I/O port addresses 0x40-0x43.
//-----------------------------------------------------------------------------
void setfreq(const int hz)
{
   unsigned short aHz = 1193180 / hz; // clocked at 1.19MHz
   IOPort aPort (0x43);
   aPort.write ((unsigned char)0xb6); // timer 2, square wave
   IOPort aFreqPort (0x42);
   aFreqPort.write((unsigned char)aHz);
   aFreqPort.write((unsigned char)(aHz >> 8));

} // end setfreq (..)


//-----------------------------------------------------------------------------
//  Pass a note, in half steps relative to 400 Hz.  The 12 step scale
//is an exponential thing.  The speaker control is at port 0x61.
//Setting the lowest two bits enables timer 2 of the 8253/8254 timer
//and turns on the speaker.
//-----------------------------------------------------------------------------
void playnote(NOTE note)
{
   IOPort aPort (0x61);
   aPort.write ((unsigned char)(aPort.readChar() | 0x03)); // start speaker going
   setfreq((int)(400 * pow(2, note.pitch / 12.0)));
   OSInterface::osinterface().sleep (note.duration*1000);
   aPort.write ((unsigned char)(aPort.readChar() &  ~0x03)); // stop speaker 
} // end playnote (..)


//-----------------------------------------------------------------------------
void main ()
{
try
{

for (int i=0; i < sizeof(notes)/sizeof(int); ++i)
{
   playnote(notes[i]);
} // endfor

} // endtry
catch ( exception & exc)
{
  cerr << exc.what() << endl;
} // endcatch
} // end main ()